@@ -4,6 +4,13 @@
 # License: 3-clause BSD, <https://github.com/smarie/python-pytest-cases/blob/master/LICENSE>
 from __future__ import division
 
+import inspect
+import sys
+import os
+from importlib import import_module
+
+from makefun import add_signature_parameters, wraps
+
 try:  # python 3.3+
     from inspect import signature, Parameter
 except ImportError:
@@ -22,13 +29,14 @@ from _pytest.python import Metafunc
 
 from .common_mini_six import string_types
 from .common_others import get_function_host
-from .common_pytest_marks import make_marked_parameter_value, get_param_argnames_as_list, has_pytest_param, \
-    get_pytest_parametrize_marks
-from .common_pytest_lazy_values import is_lazy_value
+from .common_pytest_marks import make_marked_parameter_value, get_param_argnames_as_list, \
+    get_pytest_parametrize_marks, get_pytest_usefixture_marks, PYTEST3_OR_GREATER, PYTEST6_OR_GREATER, \
+    PYTEST38_OR_GREATER, PYTEST34_OR_GREATER, PYTEST33_OR_GREATER, PYTEST32_OR_GREATER
+from .common_pytest_lazy_values import is_lazy_value, is_lazy
 
 
 # A decorator that will work to create a fixture containing 'yield', whatever the pytest version, and supports hooks
-if LooseVersion(pytest.__version__) >= LooseVersion('3.0.0'):
+if PYTEST3_OR_GREATER:
     def pytest_fixture(hook=None, **kwargs):
         def _decorate(f):
             # call hook if needed
@@ -58,6 +66,18 @@ else:
         return _decorate
 
 
+def pytest_is_running():
+    """Return True if the current process is a pytest run
+
+    See https://stackoverflow.com/questions/25188119/test-if-code-is-executed-from-within-a-py-test-session
+    """
+    if PYTEST32_OR_GREATER:
+        return "PYTEST_CURRENT_TEST" in os.environ
+    else:
+        import re
+        return any(re.findall(r'pytest|py.test', sys.argv[0]))
+
+
 def remove_duplicates(lst):
     dset = set()
     # relies on the fact that dset.add() always returns None.
@@ -81,6 +101,31 @@ def is_fixture(fixture_fun  # type: Any
         return False
 
 
+def list_all_fixtures_in(cls_or_module, return_names=True, recurse_to_module=False):
+    """
+    Returns a list containing all fixture names (or symbols if `return_names=False`)
+    in the given class or module.
+
+    Note that `recurse_to_module` can be used so that the fixtures in the parent
+    module of a class are listed too.
+
+    :param cls_or_module:
+    :param return_names:
+    :param recurse_to_module:
+    :return:
+    """
+    res = [get_fixture_name(symb) if return_names else symb
+           for n, symb in inspect.getmembers(cls_or_module, lambda f: inspect.isfunction(f) or inspect.ismethod(f))
+           if is_fixture(symb)]
+
+    if recurse_to_module and not inspect.ismodule(cls_or_module):
+        # TODO currently this only works for a single level of nesting, we should use __qualname__ (py3) or .im_class
+        host = import_module(cls_or_module.__module__)
+        res += list_all_fixtures_in(host, recurse_to_module=True, return_names=return_names)
+
+    return res
+
+
 def safe_isclass(obj  # type: object
                  ):
     # type: (...) -> bool
@@ -170,7 +215,7 @@ def get_parametrization_markers(fnode):
     :param fnode:
     :return:
     """
-    if LooseVersion(pytest.__version__) >= LooseVersion('3.4.0'):
+    if PYTEST34_OR_GREATER:
         return list(fnode.iter_markers(name="parametrize"))
     else:
         return list(fnode.parametrize)
@@ -223,10 +268,8 @@ def make_test_ids(global_ids, id_marks, argnames=None, argvalues=None, precomput
     """
     if global_ids is not None:
         # overridden at global pytest.mark.parametrize level - this takes precedence.
-        try:  # an explicit list of ids ?
-            p_ids = list(global_ids)
-        except TypeError:  # a callable to apply on the values
-            p_ids = list(global_ids(v) for v in argvalues)
+        # resolve possibly infinite generators of ids here
+        p_ids = resolve_ids(global_ids, argvalues, full_resolve=True)
     else:
         # default: values-based
         if precomputed_ids is not None:
@@ -243,6 +286,48 @@ def make_test_ids(global_ids, id_marks, argnames=None, argvalues=None, precomput
     return p_ids
 
 
+def resolve_ids(ids,                # type: Optional[Union[Callable, Iterable[str]]]
+                argvalues,          # type: Sized(Any)
+                full_resolve=False  # type: bool
+                ):
+    # type: (...) -> Union[List[str], Callable]
+    """
+    Resolves the `ids` argument of a parametrized fixture.
+
+    If `full_resolve` is False (default), iterable ids will be resolved, but not callable ids. This is useful if the
+    `argvalues` have not yet been cleaned of possible `pytest.param` wrappers.
+
+    If `full_resolve` is True, callable ids will be called using the argvalues, so the result is guaranteed to be a
+    list.
+    """
+    try:
+        # an explicit list or generator of ids ?
+        iter(ids)
+    except TypeError:
+        # a callable to apply on the values
+        if full_resolve:
+            return [ids(v) for v in argvalues]
+        else:
+            # return the callable without resolving
+            return ids
+    else:
+        # iterable.
+        try:
+            # a sized container ? (list, set, tuple)
+            nb_ids = len(ids)
+            # convert to list
+            ids = list(ids)
+        except TypeError:
+            # a generator. Consume it
+            ids = [id for id, v in zip(ids, argvalues)]
+            nb_ids = len(ids)
+
+        if nb_ids != len(argvalues):
+            raise ValueError("Explicit list or generator of `ids` provided has a different length (%s) than the number "
+                             "of argvalues (%s). Ids provided: %r" % (len(ids), len(argvalues), ids))
+        return ids
+
+
 def make_test_ids_from_param_values(param_names,
                                     param_values,
                                     ):
@@ -277,37 +362,37 @@ def make_test_ids_from_param_values(param_names,
 
 
 # ---- ParameterSet api ---
-def analyze_parameter_set(pmark=None, argnames=None, argvalues=None, ids=None, check_nb=True):
-    """
-    analyzes a parameter set passed either as a pmark or as distinct
-    (argnames, argvalues, ids) to extract/construct the various ids, marks, and
-    values
-
-    See also pytest.Metafunc.parametrize method, that calls in particular
-    pytest.ParameterSet._for_parametrize and _pytest.python._idvalset
-
-    :param pmark:
-    :param argnames:
-    :param argvalues:
-    :param ids:
-    :param check_nb: a bool indicating if we should raise an error if len(argnames) > 1 and any argvalue has
-         a different length than len(argnames)
-    :return: ids, marks, values
-    """
-    if pmark is not None:
-        if any(a is not None for a in (argnames, argvalues, ids)):
-            raise ValueError("Either provide a pmark OR the details")
-        argnames = pmark.param_names
-        argvalues = pmark.param_values
-        ids = pmark.param_ids
-
-    # extract all parameters that have a specific configuration (pytest.param())
-    custom_pids, p_marks, p_values = extract_parameterset_info(argnames, argvalues, check_nb=check_nb)
-
-    # get the ids by merging/creating the various possibilities
-    p_ids = make_test_ids(argnames=argnames, argvalues=p_values, global_ids=ids, id_marks=custom_pids)
-
-    return p_ids, p_marks, p_values
+# def analyze_parameter_set(pmark=None, argnames=None, argvalues=None, ids=None, check_nb=True):
+#     """
+#     analyzes a parameter set passed either as a pmark or as distinct
+#     (argnames, argvalues, ids) to extract/construct the various ids, marks, and
+#     values
+#
+#     See also pytest.Metafunc.parametrize method, that calls in particular
+#     pytest.ParameterSet._for_parametrize and _pytest.python._idvalset
+#
+#     :param pmark:
+#     :param argnames:
+#     :param argvalues:
+#     :param ids:
+#     :param check_nb: a bool indicating if we should raise an error if len(argnames) > 1 and any argvalue has
+#          a different length than len(argnames)
+#     :return: ids, marks, values
+#     """
+#     if pmark is not None:
+#         if any(a is not None for a in (argnames, argvalues, ids)):
+#             raise ValueError("Either provide a pmark OR the details")
+#         argnames = pmark.param_names
+#         argvalues = pmark.param_values
+#         ids = pmark.param_ids
+#
+#     # extract all parameters that have a specific configuration (pytest.param())
+#     custom_pids, p_marks, p_values = extract_parameterset_info(argnames, argvalues, check_nb=check_nb)
+#
+#     # get the ids by merging/creating the various possibilities
+#     p_ids = make_test_ids(argnames=argnames, argvalues=p_values, global_ids=ids, id_marks=custom_pids)
+#
+#     return p_ids, p_marks, p_values
 
 
 def extract_parameterset_info(argnames, argvalues, check_nb=True):
@@ -347,7 +432,7 @@ def extract_pset_info_single(nbnames, argvalue):
         # --marks
         marks = get_marked_parameter_marks(argvalue)
         # --value(a tuple if this is a tuple parameter)
-        argvalue = get_marked_parameter_values(argvalue)
+        argvalue = get_marked_parameter_values(argvalue, nbargs=nbnames)
         return _id, marks, argvalue[0] if nbnames == 1 else argvalue
     else:
         # normal argvalue
@@ -363,7 +448,8 @@ try:  # pytest 3.x+
     def get_marked_parameter_marks(v):
         return v.marks
 
-    def get_marked_parameter_values(v):
+    def get_marked_parameter_values(v, nbargs):
+        """This always returns a tuple. nbargs is useful for pytest2 compatibility """
         return v.values
 
     def get_marked_parameter_id(v):
@@ -376,7 +462,7 @@ except ImportError:  # pytest 2.x
     def ParameterSet(values,
                      id,  # noqa
                      marks):
-        """ Dummy function (not a class) used only by parametrize_plus """
+        """ Dummy function (not a class) used only by `parametrize` """
         if id is not None:
             raise ValueError("This should not happen as `pytest.param` does not exist in pytest 2")
 
@@ -398,11 +484,20 @@ except ImportError:  # pytest 2.x
     def get_marked_parameter_marks(v):
         return [v]
 
-    def get_marked_parameter_values(v):
-        if v.name in ('skip', 'skipif'):
-            return v.args[-1]  # see MetaFunc.parametrize in pytest 2 to be convinced :)
+    def get_marked_parameter_values(v, nbargs):
+        """Returns a tuple containing the values"""
+
+        # v.args[-1] contains the values.
+        # see MetaFunc.parametrize in pytest 2 to be convinced :)
+
+        # if v.name in ('skip', 'skipif'):
+        if nbargs == 1:
+            # the last element of args is not a tuple when there is a single arg.
+            return (v.args[-1],)
         else:
-            raise ValueError("Unsupported mark")
+            return v.args[-1]
+        # else:
+        #     raise ValueError("Unsupported mark")
 
     def get_marked_parameter_id(v):
         return v.kwargs.get('id', None)
@@ -432,12 +527,12 @@ def get_pytest_function_scopenum():
 
 from _pytest.python import _idval  # noqa
 
-if LooseVersion(pytest.__version__) >= LooseVersion('6.0.0'):
+if PYTEST6_OR_GREATER:
     _idval_kwargs = dict(idfn=None,
                          nodeid=None,  # item is not used in pytest(>=6.0.0) nodeid is only used by idfn
                          config=None  # if a config hook was available it would be used before this is called)
                          )
-elif LooseVersion(pytest.__version__) >= LooseVersion('3.8.0'):
+elif PYTEST38_OR_GREATER:
     _idval_kwargs = dict(idfn=None,
                          item=None,  # item is only used by idfn
                          config=None  # if a config hook was available it would be used before this is called)
@@ -465,7 +560,15 @@ def mini_idval(
 
 
 def mini_idvalset(argnames, argvalues, idx):
-    """ mimic _pytest.python._idvalset """
+    """ mimic _pytest.python._idvalset but can handle lazyvalues used for tuples or args
+
+    argvalues should not be a pytest.param (ParameterSet)
+    This function returns a SINGLE id for a single test node
+    """
+    if len(argnames) > 1 and is_lazy(argvalues):
+        # handle the case of LazyTuple used for several args
+        return argvalues.get_id()
+
     this_id = [
         _idval(val, argname, idx=idx, **_idval_kwargs)
         for val, argname in zip(argvalues, argnames)
@@ -476,8 +579,6 @@ def mini_idvalset(argnames, argvalues, idx):
 try:
     from _pytest.compat import getfuncargnames  # noqa
 except ImportError:
-    import sys
-
     def num_mock_patch_args(function):
         """ return number of arguments used up by mock arguments (if any) """
         patchings = getattr(function, "patchings", None)
@@ -527,17 +628,29 @@ class MiniFuncDef(object):
 class MiniMetafunc(Metafunc):
     # noinspection PyMissingConstructor
     def __init__(self, func):
-        self.config = None
+        from .plugin import PYTEST_CONFIG  # late import to ensure config has been loaded by now
+
+        self.config = PYTEST_CONFIG
+        if self.config is None:
+            # only raise if we are in pytest, otherwise silently skip
+            if pytest_is_running():
+                raise ValueError("Internal error - config has not been correctly loaded. Please report")
+
         self.function = func
         self.definition = MiniFuncDef(func.__name__)
         self._calls = []
         # non-default parameters
         self.fixturenames = getfuncargnames(func)
+        # add declared used fixtures with @pytest.mark.usefixtures
+        self.fixturenames_not_in_sig = [f for f in get_pytest_usefixture_marks(func) if f not in self.fixturenames]
+        if self.fixturenames_not_in_sig:
+            self.fixturenames = tuple(self.fixturenames_not_in_sig + list(self.fixturenames))
         # get parametrization marks
         self.pmarks = get_pytest_parametrize_marks(self.function)
         if self.is_parametrized:
             self.update_callspecs()
-            self.required_fixtures = set(self.fixturenames) - set(self._calls[0].funcargs)
+            # preserve order
+            self.required_fixtures = tuple(f for f in self.fixturenames if f not in self._calls[0].funcargs)
         else:
             self.required_fixtures = self.fixturenames
 
@@ -556,20 +669,65 @@ class MiniMetafunc(Metafunc):
         """
         for pmark in self.pmarks:
             if len(pmark.param_names) == 1:
-                argvals = tuple(v if is_marked_parameter_value(v) else (v,) for v in pmark.param_values)
+                if PYTEST3_OR_GREATER:
+                    argvals = tuple(v if is_marked_parameter_value(v) else (v,) for v in pmark.param_values)
+                else:
+                    argvals = []
+                    for v in pmark.param_values:
+                        if is_marked_parameter_value(v):
+                            newmark = MarkDecorator(v.markname, v.args[:-1] + ((v.args[-1],),), v.kwargs)
+                            argvals.append(newmark)
+                        else:
+                            argvals.append((v,))
+                    argvals = tuple(argvals)
             else:
                 argvals = pmark.param_values
             self.parametrize(argnames=pmark.param_names, argvalues=argvals, ids=pmark.param_ids,
                              # use indirect = False and scope = 'function' to avoid having to implement complex patches
                              indirect=False, scope='function')
 
-        if not has_pytest_param:
-            # fix the CallSpec2 instances so that the marks appear
+        if not PYTEST33_OR_GREATER:
+            # fix the CallSpec2 instances so that the marks appear in an attribute "mark"
             # noinspection PyProtectedMember
             for c in self._calls:
                 c.marks = list(c.keywords.values())
 
 
+def add_fixture_params(func, new_names):
+    """Creates a wrapper of the given function with additional arguments"""
+
+    old_sig = signature(func)
+
+    # prepend all new parameters if needed
+    for n in new_names:
+        if n in old_sig.parameters:
+            raise ValueError("argument named %s already present in signature" % n)
+    new_sig = add_signature_parameters(old_sig,
+                                       first=[Parameter(n, kind=Parameter.POSITIONAL_OR_KEYWORD) for n in new_names])
+
+    assert not isgeneratorfunction(func)
+
+    # normal function with return statement
+    @wraps(func, new_sig=new_sig)
+    def wrapped_func(**kwargs):
+        for n in new_names:
+            kwargs.pop(n)
+        return func(**kwargs)
+
+    # else:
+    #     # generator function (with a yield statement)
+    #     @wraps(fixture_func, new_sig=new_sig)
+    #     def wrapped_fixture_func(*args, **kwargs):
+    #         request = kwargs['request'] if func_needs_request else kwargs.pop('request')
+    #         if is_used_request(request):
+    #             for res in fixture_func(*args, **kwargs):
+    #                 yield res
+    #         else:
+    #             yield NOT_USED
+
+    return wrapped_func
+
+
 def get_callspecs(func):
     """
     Returns a list of pytest CallSpec objects corresponding to calls that should be made for this parametrized function.
