@@ -2,8 +2,8 @@
 #          + All contributors to <https://github.com/smarie/python-pytest-cases>
 #
 # License: 3-clause BSD, <https://github.com/smarie/python-pytest-cases/blob/master/LICENSE>
-from distutils.version import LooseVersion
 from functools import partial
+import weakref
 
 try:  # python 3.3+
     from inspect import signature
@@ -11,13 +11,17 @@ except ImportError:
     from funcsigs import signature  # noqa
 
 try:
-    from typing import Union, Callable, List, Any, Sequence, Optional  # noqa
+    from typing import Union, Callable, List, Set, Tuple, Any, Sequence, Optional, Iterable  # noqa
 except ImportError:
     pass
 
-import pytest
+try:
+    from _pytest.mark.structures import MarkDecorator, Mark  # noqa
+except ImportError:
+    pass
 
-from .common_pytest_marks import get_pytest_marks_on_function, transform_marks_into_decorators
+from .common_pytest_marks import get_pytest_marks_on_function, markdecorators_as_tuple, PYTEST53_OR_GREATER, \
+    markdecorators_to_markinfos
 
 
 class Lazy(object):
@@ -35,8 +39,8 @@ class Lazy(object):
         raise NotImplementedError()
 
     # @abstractmethod
-    def get(self):
-        """Return the value to use by pytest"""
+    def get(self, request_or_item):
+        """Return the actual value to use by pytest in the given context"""
         raise NotImplementedError()
 
     def __str__(self):
@@ -113,24 +117,28 @@ def partial_to_str(partialfun):
     return "%s(%s)" % (partialfun.func.__name__, strargs)
 
 
-pytest53 = LooseVersion(pytest.__version__) >= LooseVersion("5.3.0")
-
-
 # noinspection PyPep8Naming
 class _LazyValue(Lazy):
     """
-    A reference to a value getter, to be used in `parametrize_plus`.
+    A reference to a value getter, to be used in `parametrize`.
 
     A `lazy_value` is the same thing than a function-scoped fixture, except that the value getter function is not a
     fixture and therefore can neither be parametrized nor depend on fixtures. It should have no mandatory argument.
+
+    The `self.get(request)` method can be used to get the value for the current pytest context. This value will
+    be cached so that plugins can call it several time without triggering new calls to the underlying function.
+    So the underlying function will be called exactly once per test node.
+
+    See https://github.com/smarie/python-pytest-cases/issues/149
+    and https://github.com/smarie/python-pytest-cases/issues/143
     """
-    if pytest53:
-        __slots__ = 'valuegetter', '_id', '_marks', 'retrieved', 'value'
+    if PYTEST53_OR_GREATER:
+        __slots__ = 'valuegetter', '_id', '_marks', 'cached_value_context', 'cached_value'
         _field_names = __slots__
     else:
         # we can not define __slots__ since we'll extend int in a subclass
         # see https://docs.python.org/3/reference/datamodel.html?highlight=__slots__#notes-on-using-slots
-        _field_names = 'valuegetter', '_id', '_marks', 'retrieved', 'value'
+        _field_names = 'valuegetter', '_id', '_marks', 'cached_value_context', 'cached_value'
 
     @classmethod
     def copy_from(cls,
@@ -139,38 +147,47 @@ class _LazyValue(Lazy):
         """Creates a copy of this _LazyValue"""
         new_obj = cls(valuegetter=obj.valuegetter, id=obj._id, marks=obj._marks)
         # make sure the copy will not need to retrieve the result if already done
-        new_obj.retrieved = obj.retrieved
-        if new_obj.retrieved:
-            new_obj.value = obj.value
+        new_obj.cached_value_context = obj.cached_value_context
+        new_obj.cached_value = obj.cached_value
         return new_obj
 
     # noinspection PyMissingConstructor
     def __init__(self,
                  valuegetter,  # type: Callable[[], Any]
                  id=None,      # type: str  # noqa
-                 marks=()      # type: Union[Any, Sequence[Any]]
+                 marks=None,   # type: Union[MarkDecorator, Iterable[MarkDecorator]]
                  ):
         self.valuegetter = valuegetter
         self._id = id
-        if isinstance(marks, (tuple, list, set)):
-            self._marks = marks
-        else:
-            self._marks = (marks, )
-        self.retrieved = False
-        self.value = None
+        self._marks = markdecorators_as_tuple(marks)
+        self.cached_value_context = None
+        self.cached_value = None
 
-    def get_marks(self, as_decorators=False):
+    def __hash__(self):
+        """Provide a minimal hash representing the class, valuegetter, id and marks"""
+        return hash((self.__class__, self.valuegetter, self._id, self._marks))
+
+    def get_marks(self,
+                  as_decorators=False  # type: bool
+                  ):
+        # type: (...) -> Union[Tuple[Mark, ...], Tuple[MarkDecorator, ...]]
         """
         Overrides default implementation to return the marks that are on the case function
 
-        :param as_decorators: when True, the marks will be transformed into MarkDecorators before being
+        :param as_decorators: when True, the marks (MarkInfo) will be transformed into MarkDecorators before being
             returned
         :return:
         """
-        valuegetter_marks = get_pytest_marks_on_function(self.valuegetter, as_decorators=as_decorators)
+        valuegetter_marks = tuple(get_pytest_marks_on_function(self.valuegetter, as_decorators=as_decorators))
 
         if self._marks:
-            return transform_marks_into_decorators(self._marks) + valuegetter_marks
+            if as_decorators:
+                # self_marks = markinfos_to_markdecorators(self._marks, function_marks=True)
+                self_marks = self._marks
+            else:
+                self_marks = markdecorators_to_markinfos(self._marks)
+
+            return self_marks + valuegetter_marks
         else:
             return valuegetter_marks
 
@@ -192,22 +209,62 @@ class _LazyValue(Lazy):
             else:
                 return vg.__name__
 
-    def get(self):
-        """ Call the underlying value getter, then return the result value (not self). With a cache mechanism """
-        if not self.retrieved:
-            # retrieve
-            self.value = self.valuegetter()
-            self.retrieved = True
+    def get(self, request_or_item):
+        """
+        Calls the underlying value getter function `self.valuegetter` and returns the result.
 
-        return self.value
+        This result is cached to ensure that the underlying getter function is called exactly once for each
+        pytest node. Note that we do not cache across calls to preserve the pytest spirit of "no leakage
+        across test nodes" especially when the value is mutable.
+
+        See https://github.com/smarie/python-pytest-cases/issues/149
+        and https://github.com/smarie/python-pytest-cases/issues/143
+
+        :param request_or_item: the context of this call: either a pytest request or test node item.
+        """
+        node = get_test_node(request_or_item)
+
+        if not self.has_cached_value(node=node):
+            # retrieve the value by calling the function
+            self.cached_value = self.valuegetter()
+            # remember the pytest context of the call with a weak reference to avoir gc issues
+            self.cached_value_context = weakref.ref(node)
+
+        return self.cached_value
+
+    def has_cached_value(self, request_or_item=None, node=None, raise_if_no_context=True):
+        """Return True if there is a cached value in self.value correnponding to the given request
+
+        A degraded query "is there a cached value" (whatever the context) can be performed by not passing any
+        request, item or node, and switching `raise_if_no_context` to False.
+
+        :param request_or_item: the pytest request or item
+        :param node: the pytest node if it already known.
+        :param raise_if_no_context: a boolean indicating if an error should be raised if `request_or_item` and `node`
+            are both None. Default is `True`.
+        """
+        if node is None:
+            # can we get that context information from the request/item ?
+            if request_or_item is None:
+                if raise_if_no_context:
+                    raise ValueError("No request, item or node was provided: I can not tell if there is a "
+                                     "cached value for your context. Switch `raise_if_no_context=False` if"
+                                     " you wish to get a degraded answer.")
+                else:
+                    # degraded answer: just tell if the cache was populated at least once
+                    return self.cached_value_context is not None
+
+            # get node context information
+            node = get_test_node(request_or_item)
+
+        elif request_or_item is not None:
+            raise ValueError("Only one of `request_or_item` and `node` should be provided")
+
+        # True if there is a cached value context that is the same as the context of the request
+        return self.cached_value_context is not None and self.cached_value_context() is node
 
     def as_lazy_tuple(self, nb_params):
-        res = LazyTuple(self, nb_params)
-        if self.retrieved:
-            # make sure the tuple will not need to retrieve the result if already done
-            res.retrieved = True
-            res.value = self.value
-        return res
+        return LazyTuple(self, nb_params)
 
     def as_lazy_items_list(self, nb_params):
         return [v for v in self.as_lazy_tuple(nb_params)]
@@ -217,7 +274,7 @@ class _LazyTupleItem(Lazy):
     """
     An item in a Lazy Tuple
     """
-    if pytest53:
+    if PYTEST53_OR_GREATER:
         __slots__ = 'host', 'item'
         _field_names = __slots__
     else:
@@ -240,19 +297,29 @@ class _LazyTupleItem(Lazy):
         self.host = host
         self.item = item
 
+    def __hash__(self):
+        """Provide a minimal hash representing the class, host and item number"""
+        return hash((self.__class__, self.host, self.item))
+
     def __repr__(self):
         """Override the inherited method to avoid infinite recursion"""
         vals_to_display = (
             ('item', self.item),  # item number first for easier debug
-            ('tuple', self.host.value if self.host.retrieved else self.host.valuegetter),  # lazy value tuple or retrieved tuple
+            ('tuple', self.host.cached_value if self.host.has_cached_value(raise_if_no_context=False) else self.host._lazyvalue),  # lazy value tuple or cached tuple
         )
         return "%s(%s)" % (self.__class__.__name__, ", ".join("%s=%r" % (k, v) for k, v in vals_to_display))
 
     def get_id(self):
         return "%s[%s]" % (self.host.get_id(), self.item)
 
-    def get(self):
-        return self.host.force_getitem(self.item)
+    def get(self, request_or_item):
+        """ Call the underlying value getter if needed (cache), then return the result tuple item value (not self).
+
+        See _LazyValue.get for details
+
+        :param request_or_item: the context of this call: either a pytest request or test node item.
+        """
+        return self.host.force_getitem(self.item, request_or_item)
 
 
 class LazyTuple(Lazy):
@@ -268,44 +335,61 @@ class LazyTuple(Lazy):
     In all other cases (when @parametrize is used on a test function), pytest unpacks the tuple so it directly
     manipulates the underlying LazyTupleItem instances.
     """
-    __slots__ = 'valuegetter', 'theoretical_size', 'retrieved', 'value'
+    __slots__ = '_lazyvalue', 'theoretical_size'
     _field_names = __slots__
 
     @classmethod
     def copy_from(cls,
                   obj  # type: LazyTuple
                   ):
-        new_obj = cls(valueref=obj.value, theoretical_size=obj.theoretical_size)
-        # make sure the copy will not need to retrieve the result if already done
-        new_obj.retrieved = obj.retrieved
-        if new_obj.retrieved:
-            new_obj.value = obj.value
-        return new_obj
+        # clone the inner lazy value
+        value_copy = obj._lazyvalue.clone()
+        return cls(valueref=value_copy, theoretical_size=obj.theoretical_size)
 
     # noinspection PyMissingConstructor
     def __init__(self,
-                 valueref,         # type: Union[LazyValue, Sequence]
+                 valueref,         # type: _LazyValue
                  theoretical_size  # type: int
                  ):
-        self.valuegetter = valueref
+        self._lazyvalue = valueref
         self.theoretical_size = theoretical_size
-        self.retrieved = False
-        self.value = None
+
+    def __hash__(self):
+        """Provide a minimal hash representing the class, lazy value, and theoretical size"""
+        return hash((self.__class__, self._lazyvalue, self.theoretical_size))
 
     def __len__(self):
         return self.theoretical_size
 
     def get_id(self):
         """return the id to use by pytest"""
-        return self.valuegetter.get_id()
+        return self._lazyvalue.get_id()
 
-    def get(self):
-        """ Call the underlying value getter, then return the result tuple (not self). With a cache mechanism """
-        if not self.retrieved:
-            # retrieve
-            self.value = self.valuegetter.get()
-            self.retrieved = True
-        return self.value
+    def get(self, request_or_item):
+        """ Call the underlying value getter if needed (cache), then return the result tuple value (not self).
+        See _LazyValue.get for details
+
+        :param request_or_item: the context of this call: either a pytest request or test node item.
+        """
+        return self._lazyvalue.get(request_or_item)
+
+    def has_cached_value(self, request_or_item=None, node=None, raise_if_no_context=True):
+        """Return True if there is a cached value correnponding to the given request
+
+        A degraded query "is there a cached value" (whatever the context) can be performed by not passing any
+        request, item or node, and switching `raise_if_no_context` to False.
+
+        :param request_or_item: the pytest request or item
+        :param node: the pytest node if it already known.
+        :param raise_if_no_context: a boolean indicating if an error should be raised if `request_or_item` and `node`
+            are both None. Default is `True`.
+        """
+        return self._lazyvalue.has_cached_value(request_or_item=request_or_item, node=node,
+                                                raise_if_no_context=raise_if_no_context)
+
+    @property
+    def cached_value(self):
+        return self._lazyvalue.cached_value
 
     def __getitem__(self, item):
         """
@@ -313,29 +397,28 @@ class LazyTuple(Lazy):
         a facade (a LazyTupleItem), so that pytest can store this item independently wherever needed, without
         yet calling the value getter.
         """
-        if self.retrieved:
-            # this is never called by pytest, but keep it for debugging
-            return self.value[item]
-        elif item >= self.theoretical_size:
+        if item >= self.theoretical_size:
             raise IndexError(item)
         else:
-            # do not retrieve yet: return a facade
+            # note: do not use the cache here since we do not know the context.
+            # return a facade than will be able to use the cache of the tuple
             return LazyTupleItem(self, item)
 
-    def force_getitem(self, item):
+    def force_getitem(self, item, request):
         """ Call the underlying value getter, then return self[i]. """
-        argvalue = self.get()
+        # Note: this will use the cache correctly if needed
+        argvalue = self.get(request)
         try:
             return argvalue[item]
         except TypeError as e:
             raise ValueError("(lazy_value) The parameter value returned by `%r` is not compliant with the number"
                              " of argnames in parametrization (%s). A %s-tuple-like was expected. "
                              "Returned lazy argvalue is %r and argvalue[%s] raised %s: %s"
-                             % (self.valuegetter, self.theoretical_size, self.theoretical_size,
+                             % (self._lazyvalue, self.theoretical_size, self.theoretical_size,
                                 argvalue, item, e.__class__, e))
 
 
-if pytest53:
+if PYTEST53_OR_GREATER:
     # in the latest versions of pytest, the default _idmaker returns the value of __name__ if it is available,
     # even if an object is not a class nor a function. So we do not need to use any special trick with our
     # lazy objects
@@ -395,13 +478,14 @@ else:
 
 def lazy_value(valuegetter,  # type: Callable[[], Any]
                id=None,      # type: str  # noqa
-               marks=()      # type: Union[Any, Sequence[Any]]
+               marks=()      # type: Union[MarkDecorator, Iterable[MarkDecorator]]
                ):
     """
-    Creates a reference to a value getter, to be used in `parametrize_plus`.
+    Creates a reference to a value getter, to be used in `parametrize`.
 
     A `lazy_value` is the same thing than a function-scoped fixture, except that the value getter function is not a
     fixture and therefore can neither be parametrized nor depend on fixtures. It should have no mandatory argument.
+    The underlying function will be called exactly once per test node.
 
     By default the associated id is the name of the `valuegetter` callable, but a specific `id` can be provided
     otherwise. Note that this `id` does not take precedence over custom `ids` or `idgen` passed to @parametrize.
@@ -439,8 +523,14 @@ def is_lazy(argval):
         return False
 
 
-def get_lazy_args(argval):
-    """ Possibly calls the lazy values contained in argval if needed, before returning it"""
+def get_lazy_args(argval, request_or_item):
+    """
+    Possibly calls the lazy values contained in argval if needed, before returning it.
+    Since the lazy values cache their result to ensure that their underlying function is called only once
+    per test node, the `request` argument here is mandatory.
+
+    :param request_or_item: the context of this call: either a pytest request or item
+    """
 
     try:
         _is_lazy = is_lazy(argval)
@@ -448,6 +538,20 @@ def get_lazy_args(argval):
         return argval
     else:
         if _is_lazy:
-            return argval.get()
+            return argval.get(request_or_item)
         else:
             return argval
+
+
+def get_test_node(request_or_item):
+    """
+    Return the test node, typically a _pytest.Function.
+    Provided arg may be the node already, or the pytest request
+
+    :param request_or_item:
+    :return:
+    """
+    try:
+        return request_or_item.node
+    except AttributeError:
+        return request_or_item
