@@ -2,6 +2,8 @@
 #          + All contributors to <https://github.com/smarie/python-pytest-cases>
 #
 # License: 3-clause BSD, <https://github.com/smarie/python-pytest-cases/blob/master/LICENSE>
+import itertools
+
 import warnings
 from distutils.version import LooseVersion
 
@@ -10,12 +12,37 @@ try:  # python 3.3+
 except ImportError:
     from funcsigs import signature  # noqa
 
+try:
+    from typing import Iterable, Optional, Tuple, List, Set, Union, Sequence  # noqa
+except ImportError:
+    pass
 
 import pytest
 
+try:
+    from _pytest.mark.structures import MarkDecorator, Mark  # noqa
+except ImportError:
+    from _pytest.mark import MarkDecorator, MarkInfo as Mark  # noqa
+
 from .common_mini_six import string_types
 
 
+PYTEST_VERSION = LooseVersion(pytest.__version__)
+PYTEST3_OR_GREATER = PYTEST_VERSION >= LooseVersion('3.0.0')
+PYTEST32_OR_GREATER = PYTEST_VERSION >= LooseVersion('3.2.0')
+PYTEST33_OR_GREATER = PYTEST_VERSION >= LooseVersion('3.3.0')
+PYTEST34_OR_GREATER = PYTEST_VERSION >= LooseVersion('3.4.0')
+PYTEST35_OR_GREATER = PYTEST_VERSION >= LooseVersion('3.5.0')
+PYTEST361_36X = LooseVersion('3.6.0') < PYTEST_VERSION < LooseVersion('3.7.0')
+PYTEST37_OR_GREATER = PYTEST_VERSION >= LooseVersion('3.7.0')
+PYTEST38_OR_GREATER = PYTEST_VERSION >= LooseVersion('3.8.0')
+PYTEST46_OR_GREATER = PYTEST_VERSION >= LooseVersion('4.6.0')
+PYTEST53_OR_GREATER = PYTEST_VERSION >= LooseVersion('5.3.0')
+PYTEST54_OR_GREATER = PYTEST_VERSION >= LooseVersion('5.4.0')
+PYTEST421_OR_GREATER = PYTEST_VERSION >= LooseVersion('4.2.1')
+PYTEST6_OR_GREATER = PYTEST_VERSION >= LooseVersion('6.0.0')
+
+
 def get_param_argnames_as_list(argnames):
     """
     pytest parametrize accepts both coma-separated names and list/tuples.
@@ -44,9 +71,9 @@ def get_parametrize_signature():
 
 class _ParametrizationMark:
     """
-    Container for the mark information that we grab from the fixtures (`@fixture_plus`)
+    Container for the mark information that we grab from the fixtures (`@fixture`)
 
-    Represents the information required by `@fixture_plus` to work.
+    Represents the information required by `@fixture` to work.
     """
     __slots__ = "param_names", "param_values", "param_ids"
 
@@ -88,9 +115,26 @@ def copy_pytest_marks(from_f, to_f, override=False):
     to_f.pytestmark = to_marks + from_marks
 
 
-def get_pytest_marks_on_function(f, as_decorators=False):
+def filter_marks(marks,  # type: Iterable[Mark]
+                 remove  # type: str
+                 ):
+    # type: (...) -> Tuple[Mark]
     """
-    Utility to return *ALL* pytest marks (not only parametrization) applied on a function
+    Returns a tuple of all marks in `marks` that do not have a 'parametrize' name.
+
+    :param marks:
+    :param remove:
+    :return:
+    """
+    return tuple(m for m in marks if m.name != remove)
+
+
+def get_pytest_marks_on_function(f,
+                                 as_decorators=False  # type: bool
+                                 ):
+    # type: (...) -> Union[List[Mark], List[MarkDecorator]]
+    """
+    Utility to return a list of *ALL* pytest marks (not only parametrization) applied on a function
     Note that this also works on classes
 
     :param f:
@@ -109,11 +153,50 @@ def get_pytest_marks_on_function(f, as_decorators=False):
 
     # in the new version of pytest the marks have to be transformed into decorators explicitly
     if as_decorators:
-        return transform_marks_into_decorators(mks, function_marks=True)
+        return markinfos_to_markdecorators(mks, function_marks=True)
     else:
         return mks
 
 
+def get_pytest_marks_on_item(item):
+    """lists all marks on an item such as `request._pyfuncitem`"""
+    if PYTEST3_OR_GREATER:
+        return item.callspec.marks
+    else:
+        return [val for val in item.keywords.values() if isinstance(val, (MarkDecorator, Mark))]
+
+
+def get_pytest_usefixture_marks(f):
+    # pytest > 3.2.0
+    marks = getattr(f, 'pytestmark', None)
+    if marks is not None:
+        return tuple(itertools.chain.from_iterable(
+            mark.args for mark in marks if mark.name == 'usefixtures'
+        ))
+    else:
+        # older versions
+        mark_info = getattr(f, 'usefixtures', None)
+        if mark_info is not None:
+            return mark_info.args
+        else:
+            return ()
+
+
+def remove_pytest_mark(f, mark_name):
+    marks = getattr(f, 'pytestmark', None)
+    if marks is not None:
+        # pytest > 3.2.0
+        new_marks = [m for m in marks if m.name != mark_name]
+        setattr(f, 'pytestmark', new_marks)
+    else:
+        # older versions
+        try:
+            delattr(f, mark_name)
+        except AttributeError:
+            pass
+    return f
+
+
 def get_pytest_parametrize_marks(f):
     """
     Returns the @pytest.mark.parametrize marks associated with a function (and only those)
@@ -168,10 +251,12 @@ if not has_pytest_param:
                 raise TypeError("argvalues must be a tuple !")
 
             # get a decorator for each of the markinfo
-            marks_mod = transform_marks_into_decorators(marks, function_marks=False)
+            marks_mod = markinfos_to_markdecorators(marks, function_marks=False)
 
-            # decorate. Warning: the argvalue MUST be in a tuple
-            return marks_mod[0](argvalues_tuple)
+            # decorate. We need to distinguish between single value and multiple values
+            # indeed in pytest 2 a single arg passed to the decorator is passed directly
+            # (for example: @pytest.mark.skip(1) in parametrize)
+            return marks_mod[0](argvalues_tuple) if len(argvalues_tuple) > 1 else marks_mod[0](argvalues_tuple[0])
 else:
     # Otherwise pytest.param exists, it is easier
     def make_marked_parameter_value(argvalues_tuple, marks):
@@ -179,18 +264,24 @@ else:
             raise TypeError("argvalues must be a tuple !")
 
         # get a decorator for each of the markinfo
-        marks_mod = transform_marks_into_decorators(marks, function_marks=False)
+        marks_mod = markinfos_to_markdecorators(marks, function_marks=False)
 
         # decorate
         return pytest.param(*argvalues_tuple, marks=marks_mod)
 
 
-def transform_marks_into_decorators(marks, function_marks=False):
+def markinfos_to_markdecorators(marks,                # type: Iterable[Mark]
+                                function_marks=False  # type: bool
+                                ):
+    # type: (...) -> List[MarkDecorator]
     """
-    Transforms the provided marks (MarkInfo) obtained from marked cases, into MarkDecorator so that they can
-    be re-applied to generated pytest parameters in the global @pytest.mark.parametrize.
+    Transforms the provided marks (MarkInfo or Mark in recent pytest) obtained from marked cases, into MarkDecorator so
+    that they can be re-applied to generated pytest parameters in the global @pytest.mark.parametrize.
+
+    Returns a list.
 
     :param marks:
+    :param function_marks:
     :return:
     """
     marks_mod = []
@@ -199,9 +290,10 @@ def transform_marks_into_decorators(marks, function_marks=False):
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
             for m in marks:
+                # create a dummy new MarkDecorator named "MarkDecorator" for reference
                 md = pytest.mark.MarkDecorator()
 
-                if LooseVersion(pytest.__version__) >= LooseVersion('3.0.0'):
+                if PYTEST3_OR_GREATER:
                     if isinstance(m, type(md)):
                         # already a decorator, we can use it
                         marks_mod.append(m)
@@ -226,3 +318,34 @@ def transform_marks_into_decorators(marks, function_marks=False):
     except Exception as e:
         warnings.warn("Caught exception while trying to mark case: [%s] %s" % (type(e), e))
     return marks_mod
+
+
+def markdecorators_as_tuple(marks  # type: Optional[Union[MarkDecorator, Iterable[MarkDecorator]]]
+                            ):
+    # type: (...) -> Tuple[MarkDecorator, ...]
+    """
+    Internal routine used to normalize marks received from users in a `marks=` parameter
+
+    :param marks:
+    :return:
+    """
+    if marks is None:
+        return ()
+
+    try:
+        # iterable ?
+        return tuple(marks)
+    except TypeError:
+        # single
+        return (marks,)
+
+
+def markdecorators_to_markinfos(marks  # type: Sequence[MarkDecorator]
+                                ):
+    # type: (...) -> Tuple[Mark, ...]
+    if PYTEST3_OR_GREATER:
+        return tuple(m.mark for m in marks)
+    elif len(marks) == 0:
+        return ()
+    else:
+        return tuple(Mark(m.name, m.args, m.kwargs) for m in marks)
