@@ -24,7 +24,7 @@ except ImportError:
 
 from .common_mini_six import string_types
 from .common_pytest import get_fixture_name, is_marked_parameter_value, get_marked_parameter_values, pytest_fixture, \
-    extract_parameterset_info, get_param_argnames_as_list, get_fixture_scope
+    extract_parameterset_info, get_param_argnames_as_list, get_fixture_scope, resolve_ids
 from .fixture__creation import get_caller_module, check_name_available, WARN
 
 
@@ -33,30 +33,48 @@ class _NotUsed:
         return "pytest_cases.NOT_USED"
 
 
+class _Used:
+    def __repr__(self):
+        return "pytest_cases.USED"
+
+
 NOT_USED = _NotUsed()
 """Object representing a fixture value when the fixture is not used"""
 
 
+USED = _Used()
+"""Object representing a fixture value when the fixture is used"""
+
+
 class UnionIdMakers(object):
     """
     The enum defining all possible id styles for union fixture parameters ("alternatives")
     """
     @classmethod
-    def nostyle(cls, param):
-        return param.alternative_name
+    def nostyle(cls,
+                param  # type: UnionFixtureAlternative
+                ):
+        """ ids are <fixture_name> """
+        return param.get_alternative_id()
 
     @classmethod
-    def explicit(cls, param):
-        return "%s_is_%s" % (param.union_name, param.alternative_name)
+    def compact(cls,
+                param  # type: UnionFixtureAlternative
+                ):
+        """ ids are /<fixture_name> """
+        return "/%s" % (param.get_alternative_id(),)
 
     @classmethod
-    def compact(cls, param):
-        return "U%s" % param.alternative_name
+    def explicit(cls,
+                 param  # type: UnionFixtureAlternative
+                 ):
+        """ ids are <union_name>/<fixture_name> """
+        return "%s/%s" % (param.get_union_id(), param.get_alternative_id())
 
     @classmethod
-    def get(cls, style  # type: str
+    def get(cls, style  # type: Union[str, Callable]
             ):
-        # type: (...) -> Callable[[Any], str]
+        # type: (...) -> Callable[[UnionFixtureAlternative], str]
         """
         Returns a function that one can use as the `ids` argument in parametrize, applying the given id style.
         See https://github.com/smarie/python-pytest-cases/issues/41
@@ -64,32 +82,57 @@ class UnionIdMakers(object):
         :param style:
         :return:
         """
-        style = style or 'nostyle'
-        try:
-            return getattr(cls, style)
-        except AttributeError:
-            raise ValueError("Unknown style: %r" % style)
+        if style is None or isinstance(style, string_types):
+            # return one of the styles from the class
+            style = style or 'nostyle'
+            try:
+                return getattr(cls, style)
+            except AttributeError:
+                raise ValueError("Unknown style: %r" % style)
+        else:
+            # assume a callable: return it directly
+            return style
 
 
 class UnionFixtureAlternative(object):
     """Defines an "alternative", used to parametrize a fixture union"""
-    __slots__ = 'union_name', 'alternative_name'
+    __slots__ = 'union_name', 'alternative_name', 'alternative_index'
 
     def __init__(self,
-                 union_name,
-                 alternative_name,
+                 union_name,        # type: str
+                 alternative_name,  # type: str
+                 alternative_index  # type: int
                  ):
+        """
+
+        :param union_name: the name of the union fixture
+        :param alternative_name: the name of the fixture that will be used by the union fixture when this alternative
+            is active
+        :param alternative_index: the index of the alternative, used for ids generation
+        """
         self.union_name = union_name
         self.alternative_name = alternative_name
+        self.alternative_index = alternative_index
 
-    # def __str__(self):
-    #     # although this would be great to have a default id directly, it may be
-    #     # very confusion for debugging so I prefer that we use id_maker('none')
-    #     # to make this default behaviour explicit and not pollute the debugging process
-    #     return self.alternative_name
+    def get_union_id(self):
+        """Used by the id makers"""
+        return self.union_name
+
+    def get_alternative_idx(self):
+        """Used by the id makers"""
+        return self.alternative_index
+
+    def get_alternative_id(self):
+        """Used by the id makers to get the minimal (no style) id. Defaults to the alternative name"""
+        return self.alternative_name
+
+    def __str__(self):
+        # This string representation can be used as an id if you pass `ids=str` to fixture_union for example
+        return "%s/%s/%s" % (self.get_union_id(), self.get_alternative_idx(), self.get_alternative_id())
 
     def __repr__(self):
-        return "%s<%s=%s>" % (self.__class__.__name__, self.union_name, self.alternative_name)
+        return "%s(union_name=%s, alternative_index=%s, alternative_name=%s)" \
+               % (self.__class__.__name__, self.union_name, self.alternative_index, self.alternative_name)
 
     @staticmethod
     def to_list_of_fixture_names(alternatives_lst  # type: List[UnionFixtureAlternative]
@@ -97,7 +140,7 @@ class UnionFixtureAlternative(object):
         res = []
         for f in alternatives_lst:
             if is_marked_parameter_value(f):
-                f = get_marked_parameter_values(f)[0]
+                f = get_marked_parameter_values(f, nbargs=1)[0]
             res.append(f.alternative_name)
         return res
 
@@ -113,7 +156,8 @@ class InvalidParamsList(Exception):
         self.params = params
 
     def __str__(self):
-        return "Invalid parameters list (`argvalues`) in pytest parametrize: %s" % self.params
+        return "Invalid parameters list (`argvalues`) in pytest parametrize. `list(argvalues)` returned an error. " \
+               "Please make sure that `argvalues` is a list, tuple or iterable : %r" % self.params
 
 
 def is_fixture_union_params(params):
@@ -134,7 +178,7 @@ def is_fixture_union_params(params):
                 return False
             p0 = params[0]
             if is_marked_parameter_value(p0):
-                p0 = get_marked_parameter_values(p0)[0]
+                p0 = get_marked_parameter_values(p0, nbargs=1)[0]
             return isinstance(p0, UnionFixtureAlternative)
     except:  # noqa
         # be conservative
@@ -175,8 +219,8 @@ def ignore_unused(fixture_func):
     # add request if needed
     func_needs_request = 'request' in old_sig.parameters
     if not func_needs_request:
-        new_sig = add_signature_parameters(old_sig,
-                                           first=Parameter('request', kind=Parameter.POSITIONAL_OR_KEYWORD))
+        # Add it last so that `self` argument in class functions remains the first
+        new_sig = add_signature_parameters(old_sig, last=Parameter('request', kind=Parameter.POSITIONAL_OR_KEYWORD))
     else:
         new_sig = old_sig
 
@@ -207,8 +251,8 @@ def ignore_unused(fixture_func):
 def fixture_union(name,                # type: str
                   fixtures,            # type: Iterable[Union[str, Callable]]
                   scope="function",    # type: str
-                  idstyle='explicit',  # type: Optional[str]
-                  ids=None,            # type: Union[Callable, List[str]]
+                  idstyle='compact',   # type: Optional[Union[str, Callable]]
+                  ids=None,            # type: Union[Callable, Iterable[str]]
                   unpack_into=None,    # type: Iterable[str]
                   autouse=False,       # type: bool
                   hook=None,           # type: Callable[[Callable], Callable]
@@ -221,17 +265,22 @@ def fixture_union(name,                # type: str
     The style of test ids corresponding to the union alternatives can be changed with `idstyle`. Three values are
     allowed:
 
-     - `'explicit'` (default) favors readability,
-     - `'compact'` adds a small mark so that at least one sees which parameters are union parameters and which others
-       are normal parameters,
-     - `None` does not change the ids.
+     - `'explicit'` favors readability with names as `<union>/<alternative>`,
+     - `'compact'` (default) adds a small mark so that at least one sees which parameters are union alternatives and
+        which others are normal parameters: `/<alternative>`
+     - `None` or `'nostyle'` provides minimalistic ids : `<alternative>`
+
+    See `UnionIdMakers` class for details.
+
+    You can also pass a callable `idstyle` that will receive instances of `UnionFixtureAlternative`. For example `str`
+    leads to very explicit ids: `<union>/<idx>/<alternative>`. See `UnionFixtureAlternative` class for details.
 
     :param name: the name of the fixture to create
     :param fixtures: an array-like containing fixture names and/or fixture symbols
     :param scope: the scope of the union. Since the union depends on the sub-fixtures, it should be smaller than the
         smallest scope of fixtures referenced.
-    :param idstyle: The style of test ids corresponding to the union alternatives. One of `'explicit'` (default),
-        `'compact'`, or `None`.
+    :param idstyle: The style of test ids corresponding to the union alternatives. One of `'explicit'`, `'compact'`,
+        `'nostyle'`/`None`, or a callable (e.g. `str`) that will receive instances of `UnionFixtureAlternative`.
     :param ids: as in pytest. The default value returns the correct fixture
     :param unpack_into: an optional iterable of names, or string containing coma-separated names, for additional
         fixtures to create to represent parts of this fixture. See `unpack_fixture` for details.
@@ -260,9 +309,9 @@ def fixture_union(name,                # type: str
     # create all alternatives and reapply the marks on them
     fix_alternatives = []
     f_names_args = []
-    for _name, _id, _mark in zip(f_names, custom_pids, p_marks):
+    for _idx, (_name, _id, _mark) in enumerate(zip(f_names, custom_pids, p_marks)):
         # create the alternative object
-        alternative = UnionFixtureAlternative(union_name=name, alternative_name=_name)
+        alternative = UnionFixtureAlternative(union_name=name, alternative_name=_name, alternative_index=_idx)
 
         # remove duplicates in the fixture arguments: each is required only once by the union fixture to create
         if _name in f_names_args:
@@ -291,16 +340,16 @@ def _fixture_union(fixtures_dest,
                    fix_alternatives,      # type: Sequence[UnionFixtureAlternative]
                    unique_fix_alt_names,  # type: List[str]
                    scope="function",      # type: str
-                   idstyle="explicit",    # type: str
-                   ids=None,              # type: Union[Callable, List[str]]
+                   idstyle="compact",     # type: Optional[Union[str, Callable]]
+                   ids=None,              # type: Union[Callable, Iterable[str]]
                    autouse=False,         # type: bool
                    hook=None,             # type: Callable[[Callable], Callable]
                    caller=fixture_union,  # type: Callable
                    **kwargs):
     """
     Internal implementation for fixture_union.
-    The "alternatives" have to be created beforehand, by the caller. This allows `fixture_union` and `parametrize_plus`
-    to use the same implementation while `parametrize_plus` uses customized "alternatives" containing more information.
+    The "alternatives" have to be created beforehand, by the caller. This allows `fixture_union` and `parametrize`
+    to use the same implementation while `parametrize` uses customized "alternatives" containing more information.
 
     :param fixtures_dest:
     :param name:
@@ -338,9 +387,15 @@ def _fixture_union(fixtures_dest,
                 raise TypeError("Union Fixture %s received invalid parameter type: %s. Please report this issue."
                                 "" % (name, _alternative.__class__))
 
+    if ids is None:
+        ids = UnionIdMakers.get(idstyle)
+    else:
+        # resolve possibly infinite generators of ids here
+        ids = resolve_ids(ids, fix_alternatives, full_resolve=False)
+
     # finally create the fixture per se.
-    _make_fix = pytest_fixture(scope=scope, params=fix_alternatives, autouse=autouse,
-                               ids=ids or UnionIdMakers.get(idstyle), hook=hook, **kwargs)
+    _make_fix = pytest_fixture(scope=scope or "function", params=fix_alternatives, autouse=autouse,
+                               ids=ids, hook=hook, **kwargs)
     new_union_fix = _make_fix(_new_fixture)
 
     # Dynamically add fixture to caller's module as explained in https://github.com/pytest-dev/pytest/issues/2424
@@ -369,9 +424,9 @@ def unpack_fixture(argnames,  # type: str
 
     ```python
     import pytest
-    from pytest_cases import unpack_fixture, fixture_plus
+    from pytest_cases import unpack_fixture, fixture
 
-    @fixture_plus
+    @fixture
     @pytest.mark.parametrize("o", ['hello', 'world'])
     def c(o):
         return o, o[0]
