@@ -4,7 +4,6 @@
 # License: 3-clause BSD, <https://github.com/smarie/python-pytest-cases/blob/master/LICENSE>
 from __future__ import division
 
-from distutils.version import LooseVersion
 from inspect import isgeneratorfunction
 from itertools import product
 from warnings import warn
@@ -27,7 +26,8 @@ except ImportError:
 
 from .common_pytest_lazy_values import get_lazy_args
 from .common_pytest import get_pytest_parametrize_marks, make_marked_parameter_value, get_param_argnames_as_list, \
-    analyze_parameter_set, combine_ids, is_marked_parameter_value, get_marked_parameter_values, pytest_fixture
+    combine_ids, is_marked_parameter_value, pytest_fixture, resolve_ids, extract_parameterset_info, make_test_ids
+from .common_pytest_marks import PYTEST3_OR_GREATER
 from .fixture__creation import get_caller_module, check_name_available, WARN, CHANGE
 from .fixture_core1_unions import ignore_unused, is_used_request, NOT_USED, _make_unpack_fixture
 
@@ -35,7 +35,7 @@ from .fixture_core1_unions import ignore_unused, is_used_request, NOT_USED, _mak
 def param_fixture(argname,           # type: str
                   argvalues,         # type: Iterable[Any]
                   autouse=False,     # type: bool
-                  ids=None,          # type: Union[Callable, List[str]]
+                  ids=None,          # type: Union[Callable, Iterable[str]]
                   scope="function",  # type: str
                   hook=None,         # type: Callable[[Callable], Callable]
                   debug=False,       # type: bool
@@ -89,7 +89,7 @@ def _create_param_fixture(fixtures_dest,
                           argname,           # type: str
                           argvalues,         # type: Sequence[Any]
                           autouse=False,     # type: bool
-                          ids=None,          # type: Union[Callable, List[str]]
+                          ids=None,          # type: Union[Callable, Iterable[str]]
                           scope="function",  # type: str
                           hook=None,         # type: Callable[[Callable], Callable]
                           auto_simplify=False,
@@ -101,7 +101,9 @@ def _create_param_fixture(fixtures_dest,
         # Simplification: do not parametrize the fixture, it will directly return the single value
         argvalue_to_return = argvalues[0]
         if is_marked_parameter_value(argvalue_to_return):
-            argvalue_to_return = get_marked_parameter_values(argvalue_to_return)
+            # Warning in that case the argvalues should not be a pytest.param.
+            # argvalue_to_return = get_marked_parameter_values(argvalue_to_return)
+            raise ValueError("When auto_simplify=True the argvalue can not be a pytest.param")
 
         # create the fixture - set its name so that the optional hook can read it easily
         @with_signature("%s()" % argname)
@@ -111,7 +113,7 @@ def _create_param_fixture(fixtures_dest,
         if debug:
             print("Creating unparametrized fixture %r returning %r" % (argname, argvalue_to_return))
 
-        fix = fixture_plus(name=argname, scope=scope, autouse=autouse, ids=ids, hook=hook, **kwargs)(__param_fixture)
+        fix = fixture(name=argname, scope=scope, autouse=autouse, ids=ids, hook=hook, **kwargs)(__param_fixture)
     else:
         # create the fixture - set its name so that the optional hook can read it easily
         @with_signature("%s(request)" % argname)
@@ -121,8 +123,8 @@ def _create_param_fixture(fixtures_dest,
         if debug:
             print("Creating parametrized fixture %r returning %r" % (argname, argvalues))
 
-        fix = fixture_plus(name=argname, scope=scope, autouse=autouse, params=argvalues, ids=ids,
-                           hook=hook, **kwargs)(__param_fixture)
+        fix = fixture(name=argname, scope=scope, autouse=autouse, params=argvalues, ids=ids,
+                      hook=hook, **kwargs)(__param_fixture)
 
     # Dynamically add fixture to caller's module as explained in https://github.com/pytest-dev/pytest/issues/2424
     check_name_available(fixtures_dest, argname, if_name_exists=WARN, caller=param_fixture)
@@ -134,7 +136,7 @@ def _create_param_fixture(fixtures_dest,
 def param_fixtures(argnames,          # type: str
                    argvalues,         # type: Iterable[Any]
                    autouse=False,     # type: bool
-                   ids=None,          # type: Union[Callable, List[str]]
+                   ids=None,          # type: Union[Callable, Iterable[str]]
                    scope="function",  # type: str
                    hook=None,         # type: Callable[[Callable], Callable]
                    debug=False,       # type: bool
@@ -192,7 +194,7 @@ def _create_params_fixture(fixtures_dest,
                            argnames_lst,      # type: Sequence[str]
                            argvalues,         # type: Sequence[Any]
                            autouse=False,     # type: bool
-                           ids=None,          # type: Union[Callable, List[str]]
+                           ids=None,          # type: Union[Callable, Iterable[str]]
                            scope="function",  # type: str
                            hook=None,         # type: Callable[[Callable], Callable]
                            debug=False,       # type: bool
@@ -211,7 +213,7 @@ def _create_params_fixture(fixtures_dest,
     if debug:
         print("Creating parametrized 'root' fixture %r returning %r" % (root_fixture_name, argvalues))
 
-    @fixture_plus(name=root_fixture_name, autouse=autouse, scope=scope, hook=hook, **kwargs)
+    @fixture(name=root_fixture_name, autouse=autouse, scope=scope, hook=hook, **kwargs)
     @pytest.mark.parametrize(argnames, argvalues, ids=ids)
     @with_signature("%s(%s)" % (root_fixture_name, argnames))
     def _root_fixture(**_kwargs):
@@ -230,7 +232,7 @@ def _create_params_fixture(fixtures_dest,
             if debug:
                 print("Creating nonparametrized 'view' fixture %r returning %r[%s]" % (argname, root_fixture_name, _param_idx))
 
-            @fixture_plus(name=argname, scope=scope, autouse=autouse, hook=hook, **kwargs)
+            @fixture(name=argname, scope=scope, autouse=autouse, hook=hook, **kwargs)
             @with_signature("%s(%s)" % (argname, root_fixture_name))
             def _param_fixture(**_kwargs):
                 params = _kwargs.pop(root_fixture_name)
@@ -259,7 +261,7 @@ def _create_params_fixture(fixtures_dest,
 @pytest.hookimpl(optionalhook=True)
 def pytest_fixture_plus(*args,
                         **kwargs):
-    warn("`pytest_fixture_plus` is deprecated. Please use the new alias `fixture_plus`. "
+    warn("`pytest_fixture_plus` and `fixture_plus` are deprecated. Please use the new alias `fixture`. "
          "See https://github.com/pytest-dev/pytest/issues/6475", category=DeprecationWarning, stacklevel=2)
     if len(args) == 1:
         if callable(args[0]):
@@ -270,14 +272,18 @@ def pytest_fixture_plus(*args,
     return _fixture_plus
 
 
+fixture_plus = pytest_fixture_plus
+"""Deprecated too"""
+
+
 @function_decorator
-def fixture_plus(scope="function",  # type: str
-                 autouse=False,     # type: bool
-                 name=None,         # type: str
-                 unpack_into=None,  # type: Iterable[str]
-                 hook=None,         # type: Callable[[Callable], Callable]
-                 fixture_func=DECORATED,  # noqa
-                 **kwargs):
+def fixture(scope="function",        # type: str
+            autouse=False,           # type: bool
+            name=None,               # type: str
+            unpack_into=None,        # type: Iterable[str]
+            hook=None,               # type: Callable[[Callable], Callable]
+            fixture_func=DECORATED,  # noqa
+            **kwargs):
     """ decorator to mark a fixture factory function.
 
     Identical to `@pytest.fixture` decorator, except that
@@ -317,6 +323,34 @@ def fixture_plus(scope="function",  # type: str
                                   hook=hook, _caller_module_offset_when_unpack=3, **kwargs)
 
 
+class FixtureParam(object):
+    __slots__ = 'argnames',
+
+    def __init__(self, argnames):
+        self.argnames = argnames
+
+    def __repr__(self):
+        return "FixtureParam(argnames=%s)" % self.argnames
+
+
+class CombinedFixtureParamValue(object):
+    """Represents a parameter value created when @parametrize is used on a @fixture """
+    __slots__ = 'param_defs', 'argvalues',
+
+    def __init__(self,
+                 param_defs, # type: Iterable[FixtureParam]
+                 argvalues):
+        self.param_defs = param_defs
+        self.argvalues = argvalues
+
+    def iterparams(self):
+        return ((pdef.argnames, v) for pdef, v in zip(self.param_defs, self.argvalues))
+
+    def __repr__(self):
+        list_str = " ; ".join(["<%r: %s>" % (a, v) for a, v in self.iterparams()])
+        return "CombinedFixtureParamValue(%s)" % list_str
+
+
 def _decorate_fixture_plus(fixture_func,
                            scope="function",   # type: str
                            autouse=False,      # type: bool
@@ -354,7 +388,7 @@ def _decorate_fixture_plus(fixture_func,
     """
     if name is not None:
         # Compatibility for the 'name' argument
-        if LooseVersion(pytest.__version__) >= LooseVersion('3.0.0'):
+        if PYTEST3_OR_GREATER:
             # pytest version supports "name" keyword argument
             kwargs['name'] = name
         elif name is not None:
@@ -377,18 +411,22 @@ def _decorate_fixture_plus(fixture_func,
         # make the fixture union-aware
         wrapped_fixture_func = ignore_unused(fixture_func)
 
+        # resolve possibly infinite generators of ids here
+        if 'params' in kwargs and 'ids' in kwargs:
+            kwargs['ids'] = resolve_ids(kwargs['ids'], kwargs['params'], full_resolve=False)
+
         # transform the created wrapper into a fixture
         return pytest_fixture(scope=scope, autouse=autouse, hook=hook, **kwargs)(wrapped_fixture_func)
 
     else:
         if 'params' in kwargs:
             raise ValueError(
-                "With `fixture_plus` you cannot mix usage of the keyword argument `params` and of "
+                "With `fixture` you cannot mix usage of the keyword argument `params` and of "
                 "the pytest.mark.parametrize marks")
 
     # (2) create the huge "param" containing all params combined
     # --loop (use the same order to get it right)
-    params_names_or_name_combinations = []
+    param_defs = []
     params_values = []
     params_ids = []
     params_marks = []
@@ -397,14 +435,18 @@ def _decorate_fixture_plus(fixture_func,
 
         # check number of parameter names in this parameterset
         if len(pmark.param_names) < 1:
-            raise ValueError("Fixture function '%s' decorated with '@fixture_plus' has an empty parameter "
+            raise ValueError("Fixture function '%s' decorated with '@fixture' has an empty parameter "
                              "name in a @pytest.mark.parametrize mark")
 
         # remember the argnames
-        params_names_or_name_combinations.append(pmark.param_names)
+        param_defs.append(FixtureParam(pmark.param_names))
 
-        # analyse contents, extract marks and custom ids, apply custom ids
-        _paramids, _pmarks, _pvalues = analyze_parameter_set(pmark=pmark, check_nb=True)
+        # separate specific configuration (pytest.param()) from the values
+        custom_pids, _pmarks, _pvalues = extract_parameterset_info(pmark.param_names, pmark.param_values, check_nb=True)
+
+        # get the ids by merging/creating the various possibilities
+        _paramids = make_test_ids(argnames=pmark.param_names, argvalues=_pvalues,
+                                  global_ids=pmark.param_ids, id_marks=custom_pids)
 
         # Finally store the ids, marks, and values for this parameterset
         params_ids.append(_paramids)
@@ -412,19 +454,24 @@ def _decorate_fixture_plus(fixture_func,
         params_values.append(tuple(_pvalues))
 
     # (3) generate the ids and values, possibly reapplying marks
-    if len(params_names_or_name_combinations) == 1:
-        # we can simplify - that will be more readable
+    if len(param_defs) == 1:
+        # A single @parametrize : we can simplify - that will be more readable
         final_ids = params_ids[0]
         final_marks = params_marks[0]
-        final_values = list(params_values[0])
+        # note: we dot his even for a single @parametrize as it allows `current_case` to get the parameter names easily
+        final_values = [CombinedFixtureParamValue(param_defs, (v,)) for v in params_values[0]]
 
         # reapply the marks
         for i, marks in enumerate(final_marks):
             if marks is not None:
                 final_values[i] = make_marked_parameter_value((final_values[i],), marks=marks)
     else:
-        final_values = list(product(*params_values))
+        # Multiple @parametrize: since pytest does not support several, we merge them with "apparence" of several
+        # --equivalent id
         final_ids = combine_ids(product(*params_ids))
+        # --merge all values, we'll unpack them in the wrapper below
+        final_values = [CombinedFixtureParamValue(param_defs, v) for v in product(*params_values)]
+
         final_marks = tuple(product(*params_marks))
 
         # reapply the marks
@@ -437,7 +484,7 @@ def _decorate_fixture_plus(fixture_func,
         raise ValueError("Internal error related to fixture parametrization- please report")
 
     # (4) wrap the fixture function so as to remove the parameter names and add 'request' if needed
-    all_param_names = tuple(v for pnames in params_names_or_name_combinations for v in pnames)
+    all_param_names = tuple(v for pnames in param_defs for v in pnames.argnames)
 
     # --create the new signature that we want to expose to pytest
     old_sig = signature(fixture_func)
@@ -449,26 +496,30 @@ def _decorate_fixture_plus(fixture_func,
     # add request if needed
     func_needs_request = 'request' in old_sig.parameters
     if not func_needs_request:
-        new_sig = add_signature_parameters(new_sig, first=Parameter('request', kind=Parameter.POSITIONAL_OR_KEYWORD))
+        # Add it last so that `self` argument in class functions remains the first
+        new_sig = add_signature_parameters(new_sig, last=Parameter('request', kind=Parameter.POSITIONAL_OR_KEYWORD))
 
     # --common routine used below. Fills kwargs with the appropriate names and values from fixture_params
     def _map_arguments(*_args, **_kwargs):
         request = _kwargs['request'] if func_needs_request else _kwargs.pop('request')
 
+        # sanity check: we have created this combined value in the combined parametrization.
+        _paramz = request.param
+        if not isinstance(_paramz, CombinedFixtureParamValue):
+            # This can happen when indirect parametrization has been used.
+            # In that case we can work but this parameter will not appear in `current_cases` fixture
+            _paramz = CombinedFixtureParamValue(param_defs, _paramz if len(param_defs) > 1 else (_paramz,))
+
         # populate the parameters
-        if len(params_names_or_name_combinations) == 1:
-            _params = [request.param]  # remove the simplification
-        else:
-            _params = request.param
-        for p_names, fixture_param_value in zip(params_names_or_name_combinations, _params):
+        for p_names, p_argvals in _paramz.iterparams():
             if len(p_names) == 1:
                 # a single parameter for that generated fixture (@pytest.mark.parametrize with a single name)
-                _kwargs[p_names[0]] = get_lazy_args(fixture_param_value)
+                _kwargs[p_names[0]] = get_lazy_args(p_argvals, request)
             else:
                 # several parameters for that generated fixture (@pytest.mark.parametrize with several names)
                 # unpack all of them and inject them in the kwargs
-                for old_p_name, old_p_value in zip(p_names, fixture_param_value):
-                    _kwargs[old_p_name] = get_lazy_args(old_p_value)
+                for old_p_name, old_p_value in zip(p_names, p_argvals):
+                    _kwargs[old_p_name] = get_lazy_args(old_p_value, request)
 
         return _args, _kwargs
 
