@@ -4,7 +4,6 @@
 # License: 3-clause BSD, <https://github.com/smarie/python-pytest-cases/blob/master/LICENSE>
 from collections import OrderedDict, namedtuple
 from copy import copy
-from distutils.version import LooseVersion
 from functools import partial
 from warnings import warn
 
@@ -28,10 +27,17 @@ except ImportError:
 
 from .common_mini_six import string_types
 from .common_pytest_lazy_values import get_lazy_args
+from .common_pytest_marks import PYTEST35_OR_GREATER, PYTEST46_OR_GREATER, PYTEST37_OR_GREATER, PYTEST54_OR_GREATER
 from .common_pytest import get_pytest_nodeid, get_pytest_function_scopenum, is_function_node, get_param_names, \
     get_param_argnames_as_list
 
-from .fixture_core1_unions import NOT_USED, is_fixture_union_params, UnionFixtureAlternative
+from .fixture_core1_unions import NOT_USED, USED, is_fixture_union_params, UnionFixtureAlternative
+
+# if PYTEST54_OR_GREATER:
+#     # we will need to clean the empty ids explicitly in the plugin :'(
+from .fixture_parametrize_plus import remove_empty_ids
+
+from .case_parametrizer_new import get_current_cases
 
 
 _DEBUG = False
@@ -61,12 +67,15 @@ def pytest_runtest_setup(item):
 
     # now item.funcargs exists so we can handle it
     if hasattr(item, "funcargs"):
-        item.funcargs = {argname: get_lazy_args(argvalue) for argname, argvalue in item.funcargs.items()}
+        item.funcargs = {argname: get_lazy_args(argvalue, item)
+                         for argname, argvalue in item.funcargs.items()}
 
 
 # @pytest.hookimpl(tryfirst=True, hookwrapper=True)
 def pytest_collection(session):
     """ HACK: override the fixture manager's `getfixtureclosure` method to replace it with ours """
+
+    # Note for reference: another way to access the fm is `metafunc.config.pluginmanager.get_plugin('funcmanage')`
     session._fixturemanager.getfixtureclosure = partial(getfixtureclosure, session._fixturemanager)  # noqa
 
 
@@ -125,11 +134,11 @@ class FixtureClosureNode(object):
         self.parent = parent_node
 
         # these will be set after closure has been built
-        self.fixture_defs = None
-        self.split_fixture_name = None
+        self.fixture_defs = None  # type: OrderedDict
+        self.split_fixture_name = None  # type: str
         self.split_fixture_alternatives = []
         # we do not use a dict any more as several children can use the same union value (doubled unions)
-        self.children = []
+        self.children = []  # type: List[FixtureClosureNode]
 
     # ------ tree ------------------
 
@@ -177,7 +186,7 @@ class FixtureClosureNode(object):
         items = self.gen_all_fixture_defs(drop_fake_fixtures=drop_fake_fixtures)
 
         # sort by scope as in pytest fixture closure creator (pytest did not do it in early versions, align with this)
-        if try_to_sort and LooseVersion(pytest.__version__) >= LooseVersion('3.5.0'):
+        if try_to_sort and PYTEST35_OR_GREATER:
             f_scope = get_pytest_function_scopenum()
             def sort_by_scope(kv_pair):  # noqa
                 fixture_name, fixture_defs = kv_pair
@@ -319,8 +328,64 @@ class FixtureClosureNode(object):
 
     # ------ tools to add new fixture names during closure construction
 
+    # def prepend_fixture_without_dependencies(self, fixname):
+    #     """"""
+    #     fixturedefs = self.fixture_defs_mgr.get_fixture_defs(fixname)
+    #     if not fixturedefs:
+    #         # fixture without definition: add it. This can happen with e.g. "requests", etc.
+    #         self.fixture_defs.insert((fixname, None))
+    #     else:
+    #         # the actual definition is the last one
+    #         _fixdef = fixturedefs[-1]
+    #         _params = _fixdef.params
+    #
+    #         if _params is not None and is_fixture_union_params(_params):
+    #             # union fixture
+    #             raise ValueError("It is not possible to add a union fixture after the initial closure has been built")
+    #         else:
+    #             # normal fixture
+    #             self.add_required_fixture(fixname, fixturedefs)
+    #
+    #             # add all dependencies in the to do list
+    #             dependencies = _fixdef.argnames
+
+    # def add_fixture_without_dependencies(self, fixname):
+    #     """Used for later addition, once the closure has been built"""
+    #     fixturedefs = self.fixture_defs_mgr.get_fixture_defs(fixname)
+    #     if not fixturedefs:
+    #         # fixture without definition: add it. This can happen with e.g. "requests", etc.
+    #         self.add_required_fixture(fixname, None)
+    #     else:
+    #         # the actual definition is the last one
+    #         _fixdef = fixturedefs[-1]
+    #         _params = _fixdef.params
+    #
+    #         if _params is not None and is_fixture_union_params(_params):
+    #             # union fixture
+    #             raise ValueError("It is not possible to add a union fixture after the initial closure has been built")
+    #         else:
+    #             # normal fixture
+    #             self.add_required_fixture(fixname, fixturedefs)
+
+    def remove_fixtures(self, fixture_names_to_remove):
+        """Remove some fixture names from all nodes in this subtree. These fixtures should not be split fixtures"""
+        _to_remove_in_children = []
+        for f in fixture_names_to_remove:
+            if self.split_fixture_name == f:
+                raise NotImplementedError("It is not currently possible to remove a split fixture name from a closure "
+                                          "with splits")
+            try:
+                del self.fixture_defs[f]
+            except KeyError:
+                _to_remove_in_children.append(f)
+
+        # propagate to children if any
+        if len(_to_remove_in_children) > 0:
+            for c in self.children:
+                c.remove_fixtures(_to_remove_in_children)
+
     def add_required_fixture(self, new_fixture_name, new_fixture_defs):
-        """Add some required fixture names to this node. Returns True if new fixtures were added here (not in child)"""
+        """Add some required fixture names to all leaves under this node"""
         if self.already_knows_fixture(new_fixture_name):
             return
         elif not self.has_split():
@@ -482,10 +547,13 @@ class SuperClosure(MutableSequence):
 
         # save the fixture closure tree root
         self.tree = root_node
+        # retrieve/sort fixture defs for quicker access
+        self._update_fixture_defs()
 
+    def _update_fixture_defs(self):
         # get a list of all fixture defs, for quicker access (and sorted)
         # sort by scope as in pytest fixture closure creator, if scope information is available
-        all_fixture_defs = root_node.get_all_fixture_defs(drop_fake_fixtures=False, try_to_sort=True)
+        all_fixture_defs = self.tree.get_all_fixture_defs(drop_fake_fixtures=False, try_to_sort=True)
 
         # # also sort all partitions (note that we cannot rely on the order in all_fixture_defs when scopes are same!)
         # if LooseVersion(pytest.__version__) >= LooseVersion('3.5.0'):
@@ -569,26 +637,60 @@ class SuperClosure(MutableSequence):
     def __delitem__(self, i):
         self.remove(self[i])
 
-    def insert(self, index, object):
-        # if index == 0:
-        #     print()
-        # elif index == len(self):
-        #     print()
-        # else:
-        #     raise ValueError("It is not possible to insert a fixture name in this closure")
-        raise NotImplementedError("When fixture unions are present, inserting a fixture in the closure is non-trivial."
-                                  " Please report this so that we can find a suitable solution for your need.")
+    def insert(self, index, fixture_name):
+        """
+        Try to transparently support inserts. Since the underlying structure is a tree, only two cases
+        are supported: inserting at position 0 and appending at position len(self).
+
+        Note that while appending has no restrictions, inserting at position 0 is only allowed for now if the
+        fixture to insert does not have a union in its associated closure.
+
+        :param index:
+        :param fixture_name:
+        :return:
+        """
+        if index == 0:
+            # build the closure associated with this new fixture name
+            fixture_defs_mgr = FixtureDefsCache(self.tree.fixture_defs_mgr.fm, self.tree.fixture_defs_mgr.nodeid)
+            closure_tree = FixtureClosureNode(fixture_defs_mgr=fixture_defs_mgr)
+            closure_tree.build_closure((fixture_name,))
+            if closure_tree.has_split():
+                raise NotImplementedError("When fixture unions are present, inserting a fixture in the closure at "
+                                          "position 0 is currently only supported if that fixture's closure does not"
+                                          "contain a union. Please report this so that we can find a suitable solution"
+                                          " for your need.")
+            else:
+                # remove those fixture definitions from all nodes in the tree
+                self.tree.remove_fixtures(closure_tree.fixture_defs.keys())
+
+                # finally prepend the defs at the beginning of the dictionnary in the first node
+                self.tree.fixture_defs = OrderedDict(list(closure_tree.fixture_defs.items())
+                                                     + list(self.tree.fixture_defs.items()))
+
+        elif index == len(self):
+            # appending is natively supported in our tree growing method
+            self.tree.build_closure((fixture_name,))
+        else:
+            raise NotImplementedError("When fixture unions are present, inserting a fixture in the closure at a "
+                                      "position different from 0 (prepend) or <end> (append) is non-trivial. Please"
+                                      "report this so that we can find a suitable solution for your need.")
+
+        # Finally update self.fixture_defs so that the "list" view reflects the changes in self.tree
+        self._update_fixture_defs()
 
     def remove(self, value):
-        # try:
-        #     del self.all_fixture_defs[value]
-        # except KeyError:
-        #     raise ValueError(value)
-        # else:
-        #     propagate the change to the tree... non-trivial
+        """
+        Try to transparently support removal. Note: since the underlying structure is a tree,
+        removing "union" fixtures is non-trivial so for now it is not supported.
 
-        raise NotImplementedError("When fixture unions are present, removing a fixture from the closure is non-trivial."
-                                  " Please report this so that we can find a suitable solution for your need.")
+        :param value:
+        :return:
+        """
+        # remove in the tree
+        self.tree.remove_fixtures((value,))
+
+        # update fixture defs
+        self._update_fixture_defs()
 
 
 def getfixtureclosure(fm, fixturenames, parentnode, ignore_args=()):
@@ -598,11 +700,11 @@ def getfixtureclosure(fm, fixturenames, parentnode, ignore_args=()):
 
     # (1) first retrieve the normal pytest output for comparison
     kwargs = dict()
-    if LooseVersion(pytest.__version__) >= LooseVersion('4.6.0'):
+    if PYTEST46_OR_GREATER:
         # new argument "ignore_args" in 4.6+
         kwargs['ignore_args'] = ignore_args
 
-    if LooseVersion(pytest.__version__) >= LooseVersion('3.7.0'):
+    if PYTEST37_OR_GREATER:
         # three outputs
         initial_names, ref_fixturenames, ref_arg2fixturedefs = \
             fm.__class__.getfixtureclosure(fm, fixturenames, parentnode, **kwargs)
@@ -619,7 +721,7 @@ def getfixtureclosure(fm, fixturenames, parentnode, ignore_args=()):
     assert set(super_closure) == set(ref_fixturenames)
     assert dict(arg2fixturedefs) == ref_arg2fixturedefs
 
-    if LooseVersion(pytest.__version__) >= LooseVersion('3.7.0'):
+    if PYTEST37_OR_GREATER:
         return _init_fixnames, super_closure, arg2fixturedefs
     else:
         return super_closure, arg2fixturedefs
@@ -646,7 +748,7 @@ def create_super_closure(fm,
         print("Creating closure for %s:" % parentid)
 
     # -- auto-use fixtures
-    _init_fixnames = fm._getautousenames(parentid)  # noqa
+    _init_fixnames = list(fm._getautousenames(parentid))  # noqa
 
     def _merge(new_items, into_list):
         """ Appends items from `new_items` into `into_list`, only if they are not already there. """
@@ -727,19 +829,22 @@ class NormalParamz(namedtuple('NormalParamz', ['argnames', 'argvalues', 'indirec
 def parametrize(metafunc, argnames, argvalues, indirect=False, ids=None, scope=None, **kwargs):
     """
     This alternate implementation of metafunc.parametrize creates a list of calls that is not just the cartesian
-    product of all parameters (like the pytest behaviour).
-
-    Instead, it offers an alternate list of calls takinginto account all union fixtures.
+    product of all parameters (like the pytest behaviour). Instead, it offers an alternate list of calls taking into
+    account all "union" fixtures.
 
     For this, it replaces the `metafunc._calls` attribute with a `CallsReactor` instance, and feeds it with all
-    parameters and parametrized fixtures independently (not doing any cross-product).
-
-    The resulting `CallsReactor` instance is then able to dynamically behave like the correct list of calls,
-    lazy-creating that list when it is used.
+    parameters and parametrized fixtures independently (not doing any cross-product during this call). The resulting
+    `CallsReactor` instance is then able to dynamically behave like the correct list of calls, lazy-creating that list
+    when it is used.
     """
     if not isinstance(metafunc.fixturenames, SuperClosure):
         # legacy method
         metafunc.__class__.parametrize(metafunc, argnames, argvalues, indirect=indirect, ids=ids, scope=scope, **kwargs)
+
+        # clean EMPTY_ID : since they are never set by us in a normal parametrize, no need to do this here.
+        # if PYTEST54_OR_GREATER:
+        #     for callspec in metafunc._calls:
+        #         remove_empty_ids(callspec)
     else:
         # get or create our special container object
         if not isinstance(metafunc._calls, CallsReactor):  # noqa
@@ -787,7 +892,7 @@ class CallsReactor(object):
 
     def __init__(self, metafunc):
         self.metafunc = metafunc
-        self._pending = []
+        self._pending = []        # type: List[Union[UnionParamz, NormalParamz]]
         self._call_list = None
 
     # -- methods to provising parametrization orders without executing them --
@@ -836,9 +941,14 @@ class CallsReactor(object):
         assert self.metafunc._calls is self
 
         # ------ parametrize the calls --------
-        # create a dictionary of pending things to parametrize, and only keep the first parameter in case of several
-        pending_items = [(get_param_argnames_as_list(p[0])[0], p) for p in self._pending]
-        pending = OrderedDict(pending_items)
+        # create a dictionary of pending fixturenames/argnames to parametrize.
+        pending_dct = OrderedDict()
+        for p in self._pending:
+            k = get_param_argnames_as_list(p[0])
+            # remember one of the argnames only so that we are able to detect where in the fixture tree the
+            # parametrization applies (it will still be applied for all of its argnames, no worries: see _process_node)
+            k = k[0]
+            pending_dct[k] = p
 
         if _DEBUG:
             print("\n---- pending parametrization ----")
@@ -851,7 +961,7 @@ class CallsReactor(object):
         assert isinstance(super_closure, SuperClosure)
 
         # Apply parametrization for calls
-        calls = get_calls_for_tree(self.metafunc, super_closure.tree, pending.copy())
+        calls = get_calls_for_tree(self.metafunc, super_closure.tree, pending_dct)
         # Alternative: use the partitions for parametrization. The issue is that this leads to a less intuitive order
         # calls = []
         # for i in range(super_closure.nb_alternative_closures):
@@ -862,6 +972,11 @@ class CallsReactor(object):
                                                                  c.id, c.funcargs, c.params)
                              for c in calls]) + "\n")
 
+        # clean EMPTY_ID set by @parametrize when there is at least a MultiParamsAlternative
+        # if PYTEST54_OR_GREATER:
+        for callspec in calls:
+            remove_empty_ids(callspec)
+
         # save the list and put back self as the _calls facade
         self._call_list = calls
         self.metafunc._calls = self
@@ -869,13 +984,30 @@ class CallsReactor(object):
         self._pending = None
 
 
-def get_calls_for_tree(metafunc, fix_closure_tree, pending):
-    calls, nodes = _process_node(metafunc, fix_closure_tree, pending.copy(), [])
-    _cleanup_calls_list(metafunc, fix_closure_tree, calls, nodes, pending)
+def get_calls_for_tree(metafunc,
+                       fix_closure_tree,  # type: FixtureClosureNode
+                       pending_dct        # type: MutableMapping[str, Union[UnionParamz, NormalParamz]]
+                       ):
+    """
+    Creates the list of calls for `metafunc` based on
+    :param metafunc:
+    :param fix_closure_tree:
+    :param pending:
+    :return:
+    """
+    pending_dct = pending_dct.copy()
+    calls, nodes_used_by_calls = _process_node(metafunc, fix_closure_tree, pending_dct, [])
+    # for each call in calls, the node in nodes_used_by_calls is the coresponding tree leaf.
+    _cleanup_calls_list(metafunc, fix_closure_tree, calls, nodes_used_by_calls, pending_dct)
     return calls
 
 
-def _cleanup_calls_list(metafunc, fix_closure_tree, calls, nodes, pending):
+def _cleanup_calls_list(metafunc,
+                        fix_closure_tree,   # type: FixtureClosureNode
+                        calls,              # type: List[CallSpec2]
+                        nodes,              # type: List[FixtureClosureNode]
+                        pending_dct         # type: MutableMapping[str, Union[UnionParamz, NormalParamz]]
+                        ):
     """
     Cleans the calls list so that all calls contain a value for all parameters. This is basically
     about adding "NOT_USED" parametrization everywhere relevant.
@@ -909,7 +1041,7 @@ def _cleanup_calls_list(metafunc, fix_closure_tree, calls, nodes, pending):
         c, n = calls[i], nodes[i]
 
         # A/ set to "not used" all parametrized fixtures that were not used in some branches
-        for fixture, p_to_apply in pending.items():
+        for fixture, p_to_apply in pending_dct.items():
             if fixture not in c.params and fixture not in c.funcargs:
                 # parametrize with a single "not used" value and discard the id
                 if isinstance(p_to_apply, UnionParamz):
@@ -943,7 +1075,7 @@ def _cleanup_calls_list(metafunc, fix_closure_tree, calls, nodes, pending):
                     c._arg2scopenum[fixture_name] = _function_scope_num  # get_pytest_scopenum(fixdef[-1].scope)  # noqa
                 else:
                     # explicitly add it as active
-                    c.params[fixture_name] = 'used'
+                    c.params[fixture_name] = USED
                     c.indices[fixture_name] = 0
                     c._arg2scopenum[fixture_name] = _function_scope_num  # get_pytest_scopenum(fixdef[-1].scope)  # noqa
 
@@ -1077,6 +1209,13 @@ def _cleanup_calls_list(metafunc, fix_closure_tree, calls, nodes, pending):
 #     return calls
 
 
+@property
+def id(self):
+    # legacy _CallSpec2 id was filtering empty strings, we'll put it back on the class below
+    # https://github.com/pytest-dev/pytest/blob/5.3.4/src/_pytest/python.py#L861
+    return "-".join(map(str, filter(None, self._idlist)))
+
+
 def _parametrize_calls(metafunc, init_calls, argnames, argvalues, discard_id=False, indirect=False, ids=None,
                        scope=None, **kwargs):
     """Parametrizes the initial `calls` with the provided information and returns the resulting new calls"""
@@ -1103,10 +1242,17 @@ def _parametrize_calls(metafunc, init_calls, argnames, argvalues, discard_id=Fal
     return new_calls
 
 
-def _process_node(metafunc, current_node, pending, calls):
+def _process_node(metafunc,
+                  current_node,  # type: FixtureClosureNode
+                  pending,       # type: MutableMapping[str, Union[UnionParamz, NormalParamz]]
+                  calls          # type: List[CallSpec2]
+                  ):
     """
-    Routine to apply all the parametrization orders in `pending` that are relevant to `current_node`,
-    to the `calls` (a list of pytest CallSpec2).
+    Routine to apply all the parametrization tasks in `pending` that are relevant to `current_node`,
+    to `calls` (a list of pytest CallSpec2).
+
+    It first applies all parametrization that correspond to current node (normal parameters),
+    then applies the "split" parametrization if needed and recurses into each tree branch.
 
     It returns a tuple containing a list of calls and a list of same length containing which leaf node each one
     corresponds to.
@@ -1119,35 +1265,25 @@ def _process_node(metafunc, current_node, pending, calls):
         the corresponding leaf node in nodes[i]
     """
 
-    # (1) first apply all non-split fixtures at this node
+    # (1) first apply all **non-split** fixtures at this node = NORMAL PARAMETERS
+    # in the order defined in the closure tree, do not trust the order of the received parametrize (`pending`)
     fixtures_at_this_node = [f for f in current_node.fixture_defs.keys()
                              if f is not current_node.split_fixture_name]
-
-    # dirty hack if we want to preserve pytest legacy order when there are no children
-    # if current_node.parent is None and not current_node.has_split():
-    #     # legacy compatibility: use pytest parametrization order even if it is wrong
-    #     # see https://github.com/pytest-dev/pytest/issues/5054
-    #
-    # else:
-    #     # rather trust the order we computed from the closure
-    #     fixtures_to_process = fixtures_at_this_node
-
     for fixturename in fixtures_at_this_node:
         try:
-            # pop it from pending - do not rely the order in pending but rather the order in the closure node
+            # pop the corresponding parametrization from pending - do not trust the order
             p_to_apply = pending.pop(fixturename)
         except KeyError:
-            # not a parametrized fixture
+            # fixturename is not a parametrized fixture, nothing to do
             continue
         else:
             if isinstance(p_to_apply, UnionParamz):
-                raise ValueError("This should not happen !")
+                raise ValueError("This should not happen! Only Normal parameters should be in fixtures_at_this_node")
             elif isinstance(p_to_apply, NormalParamz):
                 # ******** Normal parametrization **********
                 if _DEBUG:
                     print("[Node %s] Applying parametrization for NORMAL %s"
-                          "" % (current_node.to_str(with_children=False, with_discarded=False),
-                                p_to_apply.argnames))
+                          "" % (current_node.to_str(with_children=False), p_to_apply.argnames))
 
                 calls = _parametrize_calls(metafunc, calls, p_to_apply.argnames, p_to_apply.argvalues,
                                            indirect=p_to_apply.indirect, ids=p_to_apply.ids,
@@ -1155,26 +1291,26 @@ def _process_node(metafunc, current_node, pending, calls):
             else:
                 raise TypeError("Invalid parametrization type: %s" % p_to_apply.__class__)
 
-    # (2) then if there is a split apply it, otherwise return
+    # (2) then is there a "union" = a split between two sub-branches in the tree ?
     if not current_node.has_split():
+        # No split = tree leaf: return
         nodes = [current_node] * len(calls)
         return calls, nodes
     else:
+        # There is a **split** : apply its parametrization (a UNION parameter)
         try:
-            # pop it from pending - do not trust the order in pending.
+            # pop the corresponding parametrization from pending - do not trust the order
             p_to_apply = pending.pop(current_node.split_fixture_name)
         except KeyError:
-            # not a parametrized fixture
-            raise ValueError("Error: fixture union parametrization not present")
+            raise ValueError("This should not happen! fixture union parametrization missing, but this is a split node")
         else:
             if isinstance(p_to_apply, NormalParamz):
-                raise ValueError("This should not happen !")
+                raise ValueError("This should not happen! Split nodes correspond to Union parameters, not Normal ones.")
             elif isinstance(p_to_apply, UnionParamz):
                 # ******** Union parametrization **********
                 if _DEBUG:
                     print("[Node %s] Applying parametrization for UNION %s"
-                          "" % (current_node.to_str(with_children=False, with_discarded=False),
-                                p_to_apply.union_fixture_name))
+                          "" % (current_node.to_str(with_children=False), p_to_apply.union_fixture_name))
 
                 # always use 'indirect' since that's a fixture.
                 calls = _parametrize_calls(metafunc, calls, p_to_apply.union_fixture_name,
@@ -1270,6 +1406,16 @@ def pytest_addoption(parser):
     )
 
 
+# will be loaded when the pytest_configure hook below is called
+PYTEST_CONFIG = None  # type: _pytest.config.Config
+
+
+def pytest_load_initial_conftests(early_config):
+    # store the received config object for future use; see #165 #166 #196
+    global PYTEST_CONFIG
+    PYTEST_CONFIG = early_config
+
+
 # @hookspec(historic=True)
 def pytest_configure(config):
     # validate the config
@@ -1315,3 +1461,23 @@ def pytest_collection_modifyitems(session, config, items):  # noqa
     else:
         # do nothing
         yield
+
+
+@pytest.fixture
+def current_cases(request):
+    """
+    A fixture containing `get_current_cases(request)`
+
+    This is a dictionary containing all case parameters for the currently active `pytest` item.
+    For each test function argument parametrized using a `@parametrize_with_case(<argname>, ...)` this dictionary
+    contains an entry `{<argname>: (actual_id, case_function)}`. If several argnames are parametrized this way,
+    a dedicated entry will be present for each argname.
+
+    If a fixture parametrized with cases is active, the dictionary will contain an entry `{<fixturename>: <dct>}` where
+    `<dct>` is a dictionary `{<argname>: (actual_id, case_function)}`.
+
+    To get more information on a case function, you can use `get_case_id(f)`, `get_case_marks(f)`, `get_case_tags(f)`.
+    You can also use `matches_tag_query` to check if a case function matches some expectations either concerning its id
+    or its tags. See https://smarie.github.io/python-pytest-cases/#filters-and-tags
+    """
+    return get_current_cases(request)
