@@ -3,10 +3,10 @@
 #
 # License: 3-clause BSD, <https://github.com/smarie/python-pytest-cases/blob/master/LICENSE>
 import pytest
-from pytest_cases import fixture_plus, pytest_fixture_plus, fixture_union
+from pytest_cases import fixture, fixture_union
 
 
-@fixture_plus(unpack_into="a,b")
+@fixture(unpack_into="a,b")
 @pytest.mark.parametrize("o", ['hello', 'world'])
 def root1(o):
     return o, o[0]
@@ -17,7 +17,7 @@ def test_function(a, b):
     assert a in ('hello', 'world')
 
 
-@pytest_fixture_plus
+@fixture
 @pytest.mark.parametrize("o", ['yeepee', 'yay'])
 def root2(o):
     return o, o[0]
@@ -32,10 +32,11 @@ def test_function2(c, d):
 
 
 def test_synthesis(module_results_dct):
-    assert list(module_results_dct) == ['test_function[hello]',
-                                        'test_function[world]',
-                                        'test_function2[root_is_root1-hello]',
-                                        'test_function2[root_is_root1-world]',
-                                        'test_function2[root_is_root2-yeepee]',
-                                        'test_function2[root_is_root2-yay]',
-                                        ]
+    assert list(module_results_dct) == [
+        'test_function[hello]',
+        'test_function[world]',
+        'test_function2[/root1-hello]',
+        'test_function2[/root1-world]',
+        'test_function2[/root2-yeepee]',
+        'test_function2[/root2-yay]',
+    ]
