@@ -5,6 +5,8 @@
 from distutils.version import LooseVersion
 
 import pytest
+
+from pytest_cases.common_pytest_marks import PYTEST3_OR_GREATER
 from pytest_cases import fixture_union
 
 
@@ -21,8 +23,8 @@ def test_foo(u):
 
 
 def test_synthesis(module_results_dct):
-    if LooseVersion(pytest.__version__) < LooseVersion('3.0.0'):
+    if not PYTEST3_OR_GREATER:
         # the way to make ids uniques in case of duplicates was different in old pytest
-        assert list(module_results_dct) == ['test_foo[0u_is_a]', 'test_foo[1u_is_a]']
+        assert list(module_results_dct) == ['test_foo[0/a]', 'test_foo[1/a]']
     else:
-        assert list(module_results_dct) == ['test_foo[u_is_a0]', 'test_foo[u_is_a1]']
+        assert list(module_results_dct) == ['test_foo[/a0]', 'test_foo[/a1]']
